import util
import mesh
import json
import re

var wifi_used = tasmota.cmd('Wifi')['Wifi'] == 'ON'

def callback(sender, payload)
  var recieved_data = json.load(payload)

  if recieved_data != nil && recieved_data.contains('action')
    var action = recieved_data['action']

    if action == 'SEND_SENSOR_DATA'
      # todo: implement sending every x time interval do reduce energy consumption
      var sensor_data = json.load(tasmota.read_sensors())

      var moisture = util.calc_soil_moisture(sensor_data['ANALOG']['A1'])
      var battery = util.calc_battery_level(sensor_data['ANALOG']['A2'])
  
      var message = {'action': 'SEND_SENSOR_DATA', 'moisture': moisture, 'battery': battery}
  
      mesh.send(sender, json.dump(message))
    elif action == 'DEEP_SLEEP' && recieved_data.contains('time')
      var time = str(recieved_data['time'])

      if re.match('^[0-9]*$', time) != nil && int(time) > 10
        mesh.send(sender, json.dump({'action': 'DEEP_SLEEP'}))

        # todo: wait for message to be sent
        tasmota.cmd('DeepSleepTime ' + time)
      else
        mesh.send(sender, json.dump({'action': 'ERROR'}))
      end
    end
  end
end

def disable_deepsleep()
  tasmota.cmd('DeepSleepTime 0')
end

def mesh_init()
  mesh.start()
  mesh.register_callback(callback)
end

def mesh_pre_init()
  mesh.init_soft_ap_mode(1)   
  
  tasmota.set_timer(50, mesh_init)
end

def wakeup_task()
  disable_deepsleep()
  util.set_wakeup_led(1)

  if !wifi_used
    mesh_pre_init()
  else 
    mesh_init()
  end
end

if !wifi_used
  tasmota.add_rule('system#init', wakeup_task)
else
  tasmota.add_rule('system#boot', wakeup_task)
end
