import mesh
import json
import path
import re

var _CHUNK_HEADER_SIZE = 12 + 3

var _wifi_used = tasmota.cmd('Wifi')['Wifi'] == 'ON'
var _files_in_process = {}

def _check_file_transfer_finished(uuid)
  var files = path.listdir('/')
  var chunks_to_receive = _files_in_process[uuid]['num_chunks']
  var merged_file_name = _files_in_process[uuid]['name']
  var received_chunks = 0

  for file_name : files
    if file_name[0..11] == uuid
      received_chunks += 1
    end
  end

  if received_chunks == chunks_to_receive
    if path.exists(merged_file_name)
      path.remove(merged_file_name)
    end

    print('opening file ' + merged_file_name)
  
    print(path.listdir('/'))
    var merged_file = open(merged_file_name, 'a')

    for file_name : files
      if file_name[0..11] == uuid
        var part_file = open(file_name, 'r')
        merged_file.write(part_file.read())
        part_file.close()
        # path.remove(file_name)
      end 
    end

    merged_file.close()

    print(path.listdir('/'))
    print('written file ' + merged_file_name)
    _files_in_process.remove(uuid)
  end
end

def _store_chunk(payload)
  var uuid = payload[0..11]
  var chunk_id = payload[12..14]

  print('storing chunk ' + chunk_id)

  var file = open(uuid + '_' + chunk_id + '.part', 'w')

  file.write(payload[15..mesh.MAX_LEN])
  file.close()
end

def _callback(sender, payload)
  print('receive payload: ' + payload)
  var recieved_data = json.load(payload)

  if recieved_data != nil && recieved_data.contains('action')
    var action = recieved_data['action']

    if action == 'EXECUTE_COMMAND'
      mesh.send(sender, json.dump({'action': 'EXECUTE_COMMAND'}))

      tasmota.cmd(recieved_data['command'])
    elif action == 'TRANSFER_FILE'
      if recieved_data.contains('name') && recieved_data.contains('uuid') && recieved_data.contains('num_chunks')
        _files_in_process[recieved_data['uuid']] = {'name': recieved_data['name'], 'num_chunks': recieved_data['num_chunks']}

        print('send payload: ' + json.dump({'action': 'TRANSFER_FILE', 'uuid': recieved_data['uuid']}))

        mesh.send(sender, json.dump({'action': 'TRANSFER_FILE', 'uuid': recieved_data['uuid']}))
      end
    elif action == 'MESH_SCAN'
      mesh.send(sender, json.dump({'action': 'MESH_SCAN'}))
    end
  elif payload != nil && size(payload) >= _CHUNK_HEADER_SIZE
    var uuid = payload[0..11]
    var chunk_id = payload[12..14]

    if _files_in_process.contains(uuid) && re.match('^[0-9]*$', chunk_id) != nil
      _store_chunk(payload)
      _check_file_transfer_finished(uuid)

      print('send payload: ' + payload[0..14])

      mesh.send(sender, payload[0..14])
    end
  end
end

def _mesh_init()
  mesh.start()
  mesh.register_callback(_callback)
end

def _mesh_pre_init()
  mesh.init_soft_ap_mode(1)   
  
  tasmota.set_timer(50, _mesh_init)
end

def _wakeup_task()

  if !_wifi_used
    _mesh_pre_init()
  else 
    _mesh_init()
  end
end

if !_wifi_used
  tasmota.add_rule('system#init', _wakeup_task)
else
  tasmota.add_rule('system#boot', _wakeup_task)
end
